/*
 * object.c
 * Handle create and subsequent garbage collection of objects.
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>, 1996.
 */

#include "config.h"
#include "config-std.h"
#include "config-mem.h"
#include "gtypes.h"
#include "access.h"
#include "object.h"
#include "constants.h"
#include "classMethod.h"
#include "itypes.h"
#include "baseClasses.h"
#include "errors.h"
#include "exception.h"
#include "itypes.h"
#include "gc.h"

/*
 * Create a new array of data items (not objects).
 */
object*
alloc_array(int sz, int type)
{
	object* obj;
	Class* class;
	Class* elclass;

	assert(type < MAXTYPES);
	elclass = TYPE_CLASS(type);
	assert(elclass != 0);

	class = lookupArray(elclass);

	/* Allocate new object - we add an extra jint of space to avoid
	 * any possible page faults when accessing the last elements
	 */
	obj = newObject(sz*TYPE_SIZE(elclass)+sizeof(jint), class);
	obj->aasize = sz;

	return (obj);
}

object*
alloc_array_object(int sz, Class* artype)
{
	Class* eltype;
	Object* obj;
	int size;

	eltype = CLASS_ELEMENT_TYPE (artype);
	if (CLASS_IS_PRIMITIVE (eltype)) {
		size = TYPE_PRIM_SIZE(eltype);
		/* Allocate new object - we add an extra jint of space to avoid
		 * any possible page faults when accessing the last elements
		 */
		size = sz * size + sizeof(jint);
	}
	else {
		size = sz * PTR_TYPE_SIZE;
	}
	obj = newObject(size, artype);
	obj->aasize = sz;

	return (obj);
}

/*
 * Create a new array of objects.
 */
object*
alloc_objectarray(int sz, char* csig)
{
	Class* class;
	Object* obj;
	char* esig;
	
	esig = &csig[1];
	class = lookupArray(classFromSig(&esig, NULL));
	obj = newObject(sz * PTR_TYPE_SIZE, class);
	obj->aasize = sz;

	return (obj);
}

/*
 * Build a multi-dimensional array.
 */
object*
alloc_multiarray(int* dims, char* cname)
{
	object* obj;
	object** array;
	int i;

	obj = alloc_objectarray(dims[0], cname);
	if (dims[1] > 0) {
		array = (object**)(obj+1);
		for (i = 0; i < dims[0]; i++) {
			array[i] = alloc_multiarray(dims+1, cname+1);
		}
	}

	return (obj);
}
