/*
 * object.h
 * Object representation.
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>, 1996.
 */

#ifndef __object_h
#define __object_h

#include <stddef.h>
#include "gtypes.h"
#include "locks.h"
#include "gc.h"

struct _dispatchTable;
struct _classes;

struct _object {
	struct _dispatchTable*	dtable;
	uint32			aasize;
	iMux			mux;
	iCv			cv;
	/* Data follows on immediately */
};

/* Number of elements. */
#define	ARRAY_SIZE(ARRAY)	((ARRAY)->aasize)
#define	ARRAY_DATA(ARRAY)	((void*)((ARRAY)+1))
#define	OBJARRAY_DATA(ARRAY)	((object**)((ARRAY)+1))

/*
 * These bizzare casts provide various offset into the object structure.
 */
#define	OBJECT_DTABLE		((int)&(*(object*)0).dtable)
#define	OBJECT_SIZE		((int)&(*(object*)0).aasize)
#define	OBJECT_DATA		((int)(((object*)0)+1))

extern Class*	ClassClass;

#define	alloc_object(c) \
	newObject(CLASS_FSIZE(c) * sizeof(int32), (c))
#define	alloc_class() \
	(classes*)newObject(sizeof(classes)-sizeof(object), ClassClass)

struct _object*	alloc_array(int, int);
struct _object* alloc_array_object(int, struct _classes*);
struct _object*	alloc_objectarray(int, char*);
struct _object*	alloc_multiarray(int*, char*);

extern void* checked_malloc (size_t);
extern void* checked_calloc (size_t, size_t);
extern void* checked_realloc (void *, size_t);
extern void  checked_free (void *);

#endif
