//----------------------------------------------------------------------------

import java.io.*;

//----------------------------------------------------------------------------

class MyApp {

  public static void main (String[] args) {

    System.out.println("Started");

    AminoAcidSet.initialise("aa.dat");
    System.out.println("AA REPORT:\n" + AminoAcidSet.report());
    
    System.out.println("Finished");
  }

} // class MyApp

//----------------------------------------------------------------------------

class AminoAcidSet {

  // constants
  public static final int            CODON_LEN = 6;
  public static final int            NUM_BONDS = 4;

  private static AminoAcid[]         aminos;
  private static int[]               xref;     // codon->AA cross ref
  
//-----------

  // read in amino acid details etc.
  public static void initialise(String filename) {

    // get codon to AA lookup table
    int total_codons = 1 << CODON_LEN;
    xref = new int [total_codons];

    //System.out.println("AminoAcidSet.initialise: total_codons="+total_codons);

    try {

      InputStream     is = new FileInputStream(filename);
      StreamTokenizer st = new StreamTokenizer(new 
					       BufferedInputStream(is, 4000));

      st.eolIsSignificant(false);
      st.commentChar('#');
      
      int stat = st.nextToken();

      // report on token type found
      if (stat == StreamTokenizer.TT_NUMBER)
	System.out.println("AminoAcidSet.initialise: 1st token is number");
      else if (stat == StreamTokenizer.TT_WORD)
	System.out.println("AminoAcidSet.initialise: 1st token is word");
      else if (stat == StreamTokenizer.TT_EOL)
	System.out.println("AminoAcidSet.initialise: 1st token is eol");
      else if (stat == StreamTokenizer.TT_EOF)
	System.out.println("AminoAcidSet.initialise: 1st token is eof");
      else
	System.out.println("AminoAcidSet.initialise: 1st token is no known type");

      // attempt to print as string and/or number
      System.out.println("AminoAcidSet.initialise: 1st st.sval="+st.sval);
      System.out.println("AminoAcidSet.initialise: 1st st.nval="+st.nval);

      int num_aa = (int)st.nval;
      aminos = new AminoAcid [num_aa];

      System.out.println("AminoAcidSet.initialise: num_aa="+num_aa);

      for (int a=0; a < num_aa; a++) {

	st.nextToken();	String name  = st.sval;
	st.nextToken();	int num_codons = (int)st.nval;

	//System.out.println("AminoAcidSet.initialise: a="+a);
	//System.out.println("AminoAcidSet.initialise: num_codons="+num_codons);
	//System.out.println("AminoAcidSet.initialise: num_codons(str)="+st.sval);

	int[] codons = new int [num_codons];
	for (int i=0; i < num_codons; i++) {
	  st.nextToken();
	  codons[i] = (int)st.nval;
	  xref[codons[i]] = a;  // record cross ref: codon->AA
	}

	double[] bonds = new double [NUM_BONDS];
	for (int i=0; i < NUM_BONDS; i++) {
	  st.nextToken();
	  bonds[i] = st.nval;
	}

	st.nextToken();	double radius = st.nval;

	// create the AminoAcid ref
	aminos[a] = new AminoAcid(name,codons,bonds,radius);

      } // for a

      is.close();

    } catch(Exception e) { e.printStackTrace(); }

  } // initialise(filename)

//-----------

  // etc... other methods

//-----------

  public static String report() {
    StringBuffer strb = new StringBuffer();
    for (int a=0; a < aminos.length; a++) {
      strb.append("Amino acid " + a + ": ");
      strb.append(aminos[a] + "\n");
    }
    return strb.toString();
  }
    
} // class AminoAcidSet

//----------------------------------------------------------------------------

class AminoAcid {

  public String            name;
  public int[]             codons;
  public double[]          bonds;
  public double            radius;

//-----------

  AminoAcid(String n, int[] c, double[] b, double r) {
    name   = n;
    codons = c;
    bonds  = b;
    radius = r;
  }

//-----------

  public String toString() {
    StringBuffer strb = new StringBuffer();
    strb.append(name + "  codons: ");
    for (int i=0; i < codons.length; i++)
      strb.append(codons[i] + " ");
    strb.append(" bonds: ");
    for (int i=0; i < bonds.length; i++)
      strb.append(bonds[i] + " ");
    strb.append(" radius=" + radius);
    return strb.toString();
  }

} // class AminoAcid
